@@ -85,9 +85,6 @@ DATABASES = {
         'PORT': '3306',
         'PASSWORD': '',
         'HOST': 'fbdb',
-        'TEST': {
-            'NAME': 'fallball_test'
-        }
     }
 }
 
@@ -117,7 +114,7 @@ LOGGING = {
         },
         'django': {
             'handlers': ['console'],
-            'level': 'INFO',
+            'level': 'ERROR',
             'propagate': True,
         },
     },
