@@ -2,6 +2,7 @@ from django.contrib import admin
 
 from .models import Application, Client, ClientUser, Reseller
 from rest_framework.authtoken.models import Token
+from rest_framework.authtoken.admin import TokenAdmin as BaseTokenAdmin
 
 
 class BaseInline(admin.TabularInline):
@@ -64,22 +65,24 @@ class ResellerAdmin(admin.ModelAdmin):
 class ClientUserInline(BaseInline):
     model = ClientUser
     extra = 0
-    readonly_fields = ['email', 'limit', 'owner', 'password']
+    readonly_fields = ['user_id', 'email', 'limit', 'owner', 'password']
 
     def has_add_permission(self, request):
         return False
 
 
 class ClientAdmin(admin.ModelAdmin):
-    fields = ['name', 'email', 'limit', 'reseller', 'status']
-    list_display = ['name', 'email', 'get_usage', 'users', 'limit', 'reseller', 'status']
+    fields = ['name', 'email', 'postal_code', 'limit', 'reseller', 'status', 'environment',
+              'country']
+    list_display = ['name', 'email', 'postal_code', 'get_usage', 'users', 'limit', 'reseller',
+                    'status']
     readonly_fields = ['status']
     search_fields = ['name', 'reseller__name']
     inlines = [ClientUserInline, ]
 
     def get_readonly_fields(self, request, client=None):
         if client:
-            return self.readonly_fields + ['reseller', 'name', 'email']
+            return self.readonly_fields + ['reseller', 'name', 'postal_code']
         return self.readonly_fields
 
     def users(self, obj):
@@ -87,10 +90,12 @@ class ClientAdmin(admin.ModelAdmin):
 
 
 class ClientUserAdmin(admin.ModelAdmin):
-    fields = ['email', 'client', 'password', 'owner', 'usage', 'limit', 'profile_type']
-    list_display = ['email', 'client', 'password', 'usage', 'limit', 'profile_type']
-    readonly_fields = []
-    search_fields = ['email', 'client__name', 'password']
+    fields = ['user_id', 'email', 'client', 'password', 'owner', 'usage', 'limit', 'profile_type',
+              'admin', 'superadmin', ]
+    list_display = ['user_id', 'email', 'client', 'password', 'usage', 'limit', 'profile_type',
+                    'admin', 'superadmin', ]
+    readonly_fields = ['user_id']
+    search_fields = ['user_id', 'email', 'client__name', 'password']
 
     def get_readonly_fields(self, request, client_user=None):
         if client_user:
@@ -98,9 +103,14 @@ class ClientUserAdmin(admin.ModelAdmin):
         return self.readonly_fields
 
 
+class TokenAdmin(BaseTokenAdmin):
+    fields = ('key', 'user')
+
+
 admin.site.register(Application, ApplicationAdmin)
 admin.site.register(Reseller, ResellerAdmin)
 admin.site.register(Client, ClientAdmin)
 admin.site.register(ClientUser, ClientUserAdmin)
 
 admin.site.unregister(Token)
+admin.site.register(Token, TokenAdmin)
