@@ -26,7 +26,7 @@ class RequestLogMiddleware(object):
 
         if request.body:
             try:
-                self.log['request']['body'] = json.loads(request.body.decode())
+                self.log['request']['body'] = json.loads(request.body.decode('utf-8'))
             except ValueError:
                 self.log['request']['body'] = {"message": "body is not valid json"}
 
@@ -40,12 +40,18 @@ class RequestLogMiddleware(object):
         reseller_name = None
 
         if response.content and response['content-type'] == 'application/json':
-            self.log['response']['body'] = json.loads(response.content.decode())
+            self.log['response']['body'] = json.loads(response.content.decode('utf-8'))
 
-        self.log['request']['headers']['REQUEST_METHOD'] = request.META['REQUEST_METHOD'],
+        self.log['request']['headers']['REQUEST_METHOD'] = request.META['REQUEST_METHOD']
 
         if 'CONTENT_TYPE' in request.META:
-            self.log['request']['headers']['CONTENT_TYPE'] = request.META['CONTENT_TYPE'],
+            self.log['request']['headers']['CONTENT_TYPE'] = request.META['CONTENT_TYPE']
+
+        if request.body and self.log['request'].get('body') is None:
+            try:
+                self.log['request']['body'] = json.loads(request.body.decode('utf-8'))
+            except ValueError:
+                self.log['request']['body'] = {"message": "body is not valid json"}
 
         self.log['response']['headers'] = response._headers
 
@@ -61,7 +67,7 @@ class RequestLogMiddleware(object):
                     reseller_name = request.resolver_match.kwargs['reseller_name']
 
             if app_id:
-                self.log['response'] = {'app': app_id, }
+                self.log['app'] = app_id
             if reseller_name:
                 self.log['reseller'] = reseller_name
 
