@@ -1,14 +1,17 @@
-import random
 import datetime
+import random
+import uuid
 
 from django.db import models
 from django.db.models import Sum
 from django.conf import settings
 from django.utils import timezone
+from django.utils.encoding import python_2_unicode_compatible
 
 UNLIMITED = -1
 
 
+@python_2_unicode_compatible
 class Application(models.Model):
     id = models.CharField(max_length=150, primary_key=True)
     owner = models.OneToOneField(settings.AUTH_USER_MODEL)
@@ -18,6 +21,7 @@ class Application(models.Model):
         return self.id
 
 
+@python_2_unicode_compatible
 class Reseller(models.Model):
     name = models.CharField(max_length=120)
     rid = models.CharField("Reseller ID", max_length=120)
@@ -49,6 +53,7 @@ class Reseller(models.Model):
         return total or 0
 
 
+@python_2_unicode_compatible
 class Client(models.Model):
     ASYNC_PROVISION_DELAY = 120  # time to complete provisioning in async scenario
     STATUS_PROVISIONING = 'provisioning'
@@ -56,8 +61,11 @@ class Client(models.Model):
     # Instance_id contains company name and used as client id
     name = models.CharField(max_length=150)
     email = models.EmailField(blank=True, null=True, default=None)
+    postal_code = models.CharField(max_length=16, blank=True, null=True, default=None)
     is_integrated = models.BooleanField(default=False)
     creation_date = models.DateTimeField(auto_now_add=True)
+    environment = models.CharField(max_length=64, blank=True, null=True, default=None)
+    country = models.CharField(max_length=64, blank=True, null=True, default=None)
     limit = models.IntegerField()
     ready_at = models.DateTimeField(default=timezone.now)
 
@@ -73,8 +81,8 @@ class Client(models.Model):
     def save(self, *args, **kwargs):
         if not self.id and self.reseller.application.async:
             delay = Client.ASYNC_PROVISION_DELAY \
-                    + random.randint(-Client.ASYNC_PROVISION_DELAY/2,
-                                     Client.ASYNC_PROVISION_DELAY/2)
+                    + random.randint(-Client.ASYNC_PROVISION_DELAY / 2,
+                                     Client.ASYNC_PROVISION_DELAY / 2)
             self.ready_at = timezone.now() + datetime.timedelta(seconds=delay)
         super(Client, self).save(*args, **kwargs)
 
@@ -89,14 +97,17 @@ class Client(models.Model):
         """
         Calculate users amount for particular company
         """
-        return ClientUser.objects.filter(client=self).count()
+        return ClientUser.objects.filter(client=self, superadmin=False).count()
 
     def get_users_by_type(self):
         """
         Calculate number of users of each type for company
         """
-        return {k: ClientUser.objects.filter(client=self, profile_type=k).count() for k in
-                dict(ClientUser.USER_PROFILE_TYPES)}
+        types = ClientUser.objects.order_by().values_list('profile_type', flat=True).distinct()
+        users = {k: ClientUser.objects.filter(client=self,
+                                              profile_type=k,
+                                              superadmin=False).count() for k in types}
+        return {k: v for k, v in users.items() if v}
 
     @property
     def status(self):
@@ -105,23 +116,27 @@ class Client(models.Model):
 
 class ClientUser(models.Model):
     DEFAULT_PROFILE = 'default'
-    GOLD_PROFILE = 'gold'
-    USER_PROFILE_TYPES = (
-        (DEFAULT_PROFILE, 'Default'),
-        (GOLD_PROFILE, 'Gold'),
-    )
+    user_id = models.UUIDField(null=True)
     email = models.EmailField()
     owner = models.OneToOneField(settings.AUTH_USER_MODEL)
     password = models.CharField(max_length=128, blank=True)
     usage = models.IntegerField(blank=True)
+    superadmin = models.BooleanField(default=False)
     admin = models.BooleanField(default=False)
     limit = models.IntegerField()
     client = models.ForeignKey(Client)
-    profile_type = models.CharField(max_length=12, choices=USER_PROFILE_TYPES,
-                                    default=DEFAULT_PROFILE)
+    profile_type = models.CharField(max_length=128, default=DEFAULT_PROFILE)
 
     class Meta:
-        unique_together = ('client', 'email')
+        unique_together = (('client', 'email'), ('client', 'user_id'))
+
+    def save(self, *args, **kwargs):
+        if not self.pk and not self.user_id:
+            self.user_id = kwargs.get('user_id', uuid.uuid4())
+        if self.superadmin:
+            self.admin = True
+            self.limit = self.usage = 0
+        super(ClientUser, self).save(*args, **kwargs)
 
     def __str__(self):
         return self.email
