@@ -1,4 +1,5 @@
 from random import randint
+import uuid
 
 from django.conf import settings
 from django.contrib.auth import get_user_model
@@ -6,6 +7,7 @@ from rest_framework import serializers as rest_serializers
 from rest_framework.authtoken.models import Token
 from rest_framework.exceptions import ValidationError
 
+from fallballapp.error_codes import ACTIVATION_ERROR, VALIDATION_ERROR
 from fallballapp.models import Application, Client, ClientUser, Reseller
 from fallballapp.utils import get_app_username, get_jwt_token
 
@@ -111,18 +113,29 @@ class ClientSerializer(rest_serializers.HyperlinkedModelSerializer):
     users_amount = rest_serializers.SerializerMethodField()
     users_by_type = rest_serializers.SerializerMethodField()
     email = rest_serializers.EmailField(required=False)
+    postal_code = rest_serializers.CharField(required=False)
 
     class Meta:
         model = Client
         fields = (
-            'name', 'email', 'creation_date', 'users_amount', 'users_by_type', 'storage',
-            'is_integrated', 'status')
+            'name', 'email', 'postal_code', 'creation_date', 'users_amount', 'users_by_type',
+            'storage', 'is_integrated', 'status', 'country', 'environment',)
         read_only_fields = ('status',)
 
-    def validate_email(self, value):
-        if '.' in value.split('@')[0]:
-            raise rest_serializers.ValidationError("Dots are not allowed "
-                                                   "in local parts of email addresses")
+    def validate_postal_code(self, value):
+        if not value.isdigit() or len(value) != 5:
+            # ValidationError is triggered with error code and message
+            raise rest_serializers.ValidationError({
+                'code': VALIDATION_ERROR,
+                'message': "Postal code must be a 5-digit number",
+            })
+
+        if value.startswith('999'):
+            raise rest_serializers.ValidationError({
+                'code': ACTIVATION_ERROR,
+                'message': "Postal code can't start with 999",
+            })
+
         return value
 
     def create(self, validated_data):
@@ -131,6 +144,13 @@ class ClientSerializer(rest_serializers.HyperlinkedModelSerializer):
         """
         return Client.objects.create(reseller=self.initial_data['reseller'], **validated_data)
 
+    def update(self, instance, validated_data):
+        if 'country' in validated_data:
+            del validated_data['country']
+        if 'environment' in validated_data:
+            del validated_data['environment']
+        return super(self.__class__, self).update(instance, validated_data)
+
     def get_users_amount(self, obj):
         return obj.get_users_amount()
 
@@ -148,20 +168,25 @@ class StorageClientUserSerializer(rest_serializers.HyperlinkedModelSerializer):
 class ClientUserSerializer(rest_serializers.ModelSerializer):
     storage = StorageClientUserSerializer(source='*')
     admin = rest_serializers.BooleanField()
+    superadmin = rest_serializers.BooleanField()
     password = rest_serializers.CharField(required=False)
-    profile_type = rest_serializers.ChoiceField(choices=ClientUser.USER_PROFILE_TYPES,
-                                                required=False)
+    profile_type = rest_serializers.CharField(required=False)
+    email = rest_serializers.EmailField(required=True)
 
     class Meta:
         model = ClientUser
-        fields = ('email', 'password', 'storage', 'admin', 'profile_type')
+        fields = ('user_id', 'email', 'password', 'storage', 'admin', 'superadmin', 'profile_type')
 
     def create(self, validated_data):
         # Usage is random but not more than limit
         if 'usage' not in validated_data:
             validated_data['usage'] = randint(0, validated_data['limit'])
-        username = get_app_username(self.initial_data['application_id'], validated_data['email'])
+
+        if 'user_id' not in validated_data:
+            validated_data['user_id'] = uuid.uuid4()
+        username = get_app_username(self.initial_data['application_id'], validated_data['user_id'])
         user = get_user_model().objects.create_user(username=username)
+
         return ClientUser.objects.create(owner=user,
                                          client=self.initial_data['client'], **validated_data)
 
@@ -174,7 +199,7 @@ class UserAuthorizationSerializer(rest_serializers.ModelSerializer):
 
     class Meta:
         model = ClientUser
-        fields = ('email', 'password', 'storage', 'admin', 'company', 'token')
+        fields = ('user_id', 'email', 'password', 'storage', 'admin', 'company', 'token')
 
     def get_company(self, obj):
         return obj.client.name
