@@ -4,7 +4,7 @@ from django.contrib.auth import get_user_model
 from django.conf import settings
 from django.shortcuts import get_object_or_404
 from rest_framework import status
-from rest_framework.authentication import TokenAuthentication, BasicAuthentication
+from rest_framework.authentication import TokenAuthentication
 from rest_framework.decorators import detail_route
 from rest_framework.permissions import IsAuthenticated
 from rest_framework.renderers import JSONRenderer
@@ -21,6 +21,7 @@ except ImportError:
     import urlparse
     from urllib import urlencode
 
+from fallballapp.auth import EmailBasicAuthentication
 from fallballapp.middleware import logger
 from fallballapp.models import Application, Client, ClientUser, Reseller, UNLIMITED
 from fallballapp.renderers import PlainTextRenderer
@@ -220,9 +221,9 @@ class ClientUserViewSet(ModelViewSet):
     serializer_class = ClientUserSerializer
     authentication_classes = (TokenAuthentication, JSONWebTokenAuthentication)
     permission_classes = (IsAuthenticated,)
-    lookup_field = 'email'
-    # Redefine regex in order to get user email as id
-    lookup_value_regex = '[^@]+@[^@]+\.[^@/]+'
+    lookup_field = 'user_id'
+    # Redefine regex in order to get user_id as id
+    lookup_value_regex = '[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}'
 
     @get_user_context
     def create(self, *args, **kwargs):
@@ -230,9 +231,15 @@ class ClientUserViewSet(ModelViewSet):
         client = kwargs['client']
         reseller = kwargs['reseller']
 
-        if ClientUser.objects.filter(email=request.data['email'], client=kwargs['client']):
+        if 'user_id' in request.data and ClientUser.objects.filter(user_id=request.data['user_id'],
+                                                                   client=kwargs['client']):
             return Response("Such user already exists", status=status.HTTP_400_BAD_REQUEST)
 
+        if 'email' in request.data and ClientUser.objects.filter(email=request.data['email'],
+                                                                 client=kwargs['client']):
+            return Response("User with email {} already exists".format(request.data['email']),
+                            status=status.HTTP_400_BAD_REQUEST)
+
         if 'usage' in request.data:
             return Response("Usage should not be specified", status=status.HTTP_400_BAD_REQUEST)
 
@@ -243,10 +250,12 @@ class ClientUserViewSet(ModelViewSet):
             request.data['application_id'] = reseller.application.id
             if 'admin' not in request.data:
                 request.data['admin'] = False
+            if 'superadmin' not in request.data:
+                request.data['superadmin'] = False
             if 'password' in request.data:
                 request.data.pop('password')
             if 'profile_type' in request.data:
-                if request.data['profile_type'] not in dict(ClientUser.USER_PROFILE_TYPES):
+                if not request.data['profile_type']:
                     request.data['profile_type'] = ClientUser.DEFAULT_PROFILE
 
             return ModelViewSet.create(self, *args, **kwargs)
@@ -255,19 +264,20 @@ class ClientUserViewSet(ModelViewSet):
 
     @get_user_context
     def destroy(self, *args, **kwargs):
-        client_user = get_object_or_404(ClientUser, email=kwargs['email'], client=kwargs['client'])
+        client_user = get_object_or_404(ClientUser, user_id=kwargs['user_id'],
+                                        client=kwargs['client'])
         client_user.delete()
         return Response("User has been deleted", status=status.HTTP_204_NO_CONTENT)
 
     @get_user_context
     def list(self, *args, **kwargs):
-        queryset = ClientUser.objects.filter(client=kwargs['client'])
+        queryset = ClientUser.objects.filter(client=kwargs['client'], superadmin=False)
         serializer = ClientUserSerializer(queryset, many=True)
         return Response(serializer.data)
 
     @get_user_context
     def retrieve(self, *args, **kwargs):
-        queryset = ClientUser.objects.filter(client=kwargs['client'], email=kwargs['email'])
+        queryset = ClientUser.objects.filter(client=kwargs['client'], user_id=kwargs['user_id'])
         if not queryset:
             return Response("User does not exist", status=status.HTTP_404_NOT_FOUND)
         serializer = ClientUserSerializer(queryset, many=True)
@@ -276,10 +286,10 @@ class ClientUserViewSet(ModelViewSet):
     @get_user_context
     def update(self, *args, **kwargs):
         client_user = ClientUser.objects.filter(client=kwargs['client'],
-                                                email=kwargs['email']).first()
+                                                user_id=kwargs['user_id']).first()
 
         if not client_user:
-            args[0].data['email'] = kwargs['email']
+            args[0].data['user_id'] = kwargs['user_id']
             return self.create(*args, **kwargs)
 
         request = args[0]
@@ -287,7 +297,7 @@ class ClientUserViewSet(ModelViewSet):
         if 'password' in request.data:
             user = get_object_or_404(get_user_model(),
                                      username=get_app_username(kwargs['application'],
-                                                               kwargs['email']))
+                                                               kwargs['user_id']))
             user.set_password(request.data['password'])
             user.save()
             client_user.password = request.data['password']
@@ -302,15 +312,17 @@ class ClientUserViewSet(ModelViewSet):
                 client_user.usage = usage
 
             space = free_space(kwargs['client'])
-            if space < client_user.limit or client_user.usage > client_user.limit:
-                return Response("Storage limit is reached", HTTP_400_BAD_REQUEST)
+            if space < client_user.limit:
+                return Response("Client storage limit is reached", HTTP_400_BAD_REQUEST)
+            if client_user.usage > client_user.limit:
+                return Response("User's current usage is greater than the desired limit",
+                                HTTP_400_BAD_REQUEST)
 
         if 'admin' in request.data:
             client_user.admin = request.data['admin']
 
         if 'profile_type' in request.data:
-            if request.data['profile_type'] in dict(ClientUser.USER_PROFILE_TYPES):
-                client_user.profile_type = request.data['profile_type']
+            client_user.profile_type = request.data['profile_type']
 
         client_user.save()
 
@@ -335,7 +347,7 @@ class ClientUserViewSet(ModelViewSet):
                 reseller = admin.client.reseller
 
         client = Client.objects.filter(reseller=reseller, name=kwargs['client_name'])
-        clientuser = ClientUser.objects.filter(client=client, email=kwargs['email']).first()
+        clientuser = ClientUser.objects.filter(client=client, user_id=kwargs['user_id']).first()
         if not clientuser or not clientuser.owner:
             return Response("User does not exist", status=status.HTTP_404_NOT_FOUND)
 
@@ -382,7 +394,7 @@ class UsersViewSet(ModelViewSet):
     queryset = ClientUser.objects.all()
     serializer_class = UserAuthorizationSerializer
     authentication_classes = (TokenAuthentication, JSONWebTokenAuthentication,
-                              BasicAuthentication)
+                              EmailBasicAuthentication)
 
     def list(self, request, *args, **kwargs):
         queryset = ClientUser.objects.filter(owner_id=request.user.id).first()
